@@ -1,22 +1,100 @@
 """
 
-The interface for the Newsletter.
+Setting up a Newsletter, and subscribing people to it.
 
-"""
+    >>> import pprint
+    >>> class MyNewsletter(Newsletter):
+    ...     _fake_emails = []
+    ...     def send_subscribe_confirm_email(self, uid, user):
+    ...         self._fake_emails.append(("subscribe", uid, user))
+    ...     def send_unsubscribe_confirm_email(self, uid, user):
+    ...         self._fake_emails.append(("unsubscribe", uid, user))
+
+    >>> n = MyNewsletter()
+    >>> n.subscribe("bla@example.com")
+    True
+    >>> secret = n.subscribers(unconfirmed = True)[0]['secret']
+    >>> n.confirm_subscribe(email = 'bla@example.com', secret = secret)
+    True
+
+    >>> n.subscribers()
+    [{'secret': '...', 'email': 'bla@example.com', 'subscribed': True}]
 
 
-"""
+
+Setting up a NewletterSend and sending email to people on it.
+    >>> class MyNewsletterSend(NewsletterSend):
+    ...     def send_send_part(self, send_part):
+    ...         print ('sending log ' + pprint.pformat(send_part))
+    ...         return True
+
+    >>> ns = MyNewsletterSend()
+    >>> newsletter_send_id = ns.create("a subject", "this is the body of the message")
+
+Add some users to the send.
+    >>> ns.add_users(newsletter_send_id,  n.subscribers())
+    True
+
+Check that the users have been added correctly.
+
+    >>> import pprint
+    >>> len(list(ns._send_parts.items()))
+    1
+
+    >>> pprint.pprint(list(sorted([sp for sp in ns._send_parts.values()])))
+    [{'body': 'this is the body of the message',
+      'email': 'bla@example.com',
+      'full_name': '',
+      'id': '...',
+      'newsletter_send_id': '...',
+      'status': 'open',
+      'subject': 'a subject',
+      'user_id': '...'}]
+
+
+Send the newsletter.
+    >>> ns.send(newsletter_send_id)
+    sending log {'body': 'this is the body of the message',
+     'email': 'bla@example.com',
+     'full_name': '',
+     'id': '...',
+     'newsletter_send_id': '...',
+     'status': 'sending',
+     'subject': 'a subject',
+     'user_id': '...'}
+    True
+
+
+
+
+
+Mini todo list.
 
 * actions
-  - create, update, delete newsletters.
-  - select people to send email to.
-    - everyone on the list.
+  X - create, update, delete newsletters.
+  X- select people to send email to.
+    X - everyone on the list.
     - select categories of people.
   - unsubscribe through url.  Given an email, and a secret code.
   - unsubscribe through admin section.  Given an email.
+    - search for users by email.
   - subscription form
     - 
-  - confirmation email, for double opt in.  Sends them a url with a secret to click on.
+  X - confirmation email, for double opt in.  Sends them a url with a secret to click on.
+    - 
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 """
 
@@ -26,76 +104,82 @@ import time
 class Newsletter(object):
     """
 
-    We create a subclass which mocks out the sending of confirmation
-    emails.
-    >>> class MyNewsletter(Newsletter):
-    ...     _fake_emails = []
-    ...     def send_subscribe_confirm_email(self, uid, user):
-    ...         self._fake_emails.append(("subscribe", uid, user))
-    ...     def send_unsubscribe_confirm_email(self, uid, user):
-    ...         self._fake_emails.append(("unsubscribe", uid, user))
-    
+    We create a subclass which mocks out the sending of confirmation emails.
+        >>> class MyNewsletter(Newsletter):
+        ...     _fake_emails = []
+        ...     def send_subscribe_confirm_email(self, uid, user):
+        ...         self._fake_emails.append(("subscribe", uid, user))
+        ...     def send_unsubscribe_confirm_email(self, uid, user):
+        ...         self._fake_emails.append(("unsubscribe", uid, user))
+        
 
-    >>> n = MyNewsletter()
-    >>> n.subscribers()
-    []
+        >>> n = MyNewsletter()
+        >>> n.subscribers()
+        []
 
-    >>> n.subscribe("bla@example.com")
-    True
-    >>> n.subscribers(unconfirmed = True)
-    [{'secret': '...', 'email': 'bla@example.com', 'subscribed': False}]
+        >>> n.subscribe("bla@example.com")
+        True
+        >>> n.subscribers(unconfirmed = True)
+        [{'secret': '...', 'email': 'bla@example.com', 'subscribed': False}]
 
-    We add the same subscriber again, and we should not overwrite details.
-    >>> n.subscribe("bla@example.com")
-    True
+    Add the same subscriber again, and we should not overwrite details.
+        >>> n.subscribe("bla@example.com")
+        True
 
     See if we sent the confirmation email.
-    >>> n._fake_emails[0]
-    ('subscribe', '...', {'secret': '...', 'email': 'bla@example.com', 'subscribed': False})
+        >>> n._fake_emails[0]
+        ('subscribe', '...', {'secret': '...', 'email': 'bla@example.com', 'subscribed': False})
 
-    >>> n.subscribers(unconfirmed = True)
-    [{'secret': '...', 'email': 'bla@example.com', 'subscribed': False}]
+        >>> n.subscribers(unconfirmed = True)
+        [{'secret': '...', 'email': 'bla@example.com', 'subscribed': False}]
 
 
 
 
-    >>> secret = n.subscribers(unconfirmed = True)[0]['secret']
-    >>> n.confirm_subscribe(email = 'bla@example.com', secret = secret)
-    True
+        >>> secret = n.subscribers(unconfirmed = True)[0]['secret']
+        >>> n.confirm_subscribe(email = 'bla@example.com', secret = secret)
+        True
 
-    >>> n.subscribers()
-    [{'secret': '...', 'email': 'bla@example.com', 'subscribed': True}]
+        >>> n.subscribers()
+        [{'secret': '...', 'email': 'bla@example.com', 'subscribed': True}]
 
 
-    >>> n.unsubscribe(email = 'bla@example.com')
-    True
+        >>> n.unsubscribe(email = 'bla@example.com')
+        True
 
-    >>> n._fake_emails[1]
-    ('unsubscribe', '...', {'secret': '...', 'email': 'bla@example.com', 'subscribed': True})
+        >>> n._fake_emails[1]
+        ('unsubscribe', '...', {'secret': '...', 'email': 'bla@example.com', 'subscribed': True})
 
 
     Let us unsubscribe someone who is not subscribed.  Returns True anyway,
       because otherwise it could leak subscribers.
-    >>> n.unsubscribe(email = 'notsubbed@example.com')
-    True
-    
+        >>> n.unsubscribe(email = 'notsubbed@example.com')
+        True
+        
 
-    >>> n.confirm_unsubscribe(email = 'bla@example.com', secret = secret)
-    True
+        >>> n.confirm_unsubscribe(email = 'bla@example.com', secret = secret)
+        True
 
-    >>> n.subscribers()
-    []
+        >>> n.subscribers()
+        []
 
-    >>> n.subscribers(unconfirmed = True)
-    []
+        >>> n.subscribers(unconfirmed = True)
+        []
     """
 
     def __init__(self):
         """
         """
         # keyed by unique id, valued by a dict of info.
-        self._subscribers = {}
-    
+        self._subscribers = self.make_subscribers_db()
+
+    def make_subscribers_db(self):
+        """ make_subscribers_db() returns a dict like object for storing the
+                subscribers.
+        """
+        return {}
+
+
 
     def subscribe(self, email, info = {}):
         """
@@ -111,7 +195,7 @@ class Newsletter(object):
             pass
         else:
             uid = self._make_uid()
-            #user["user_id"] = uid
+            user["user_id"] = uid
             self._subscribers[uid] = user
             self.send_subscribe_confirm_email(uid, user)
 
@@ -198,92 +282,141 @@ class NewsletterSend(object):
 
 
 
-    >>> n = Newsletter()
-    >>> n.subscribe("bla@example.com")
-    True
-    >>> secret = n.subscribers(unconfirmed = True)[0]['secret']
-    >>> n.confirm_subscribe(email = 'bla@example.com', secret = secret)
-    True
-
-    >>> n.subscribers()
-    [{'secret': '...', 'email': 'bla@example.com', 'subscribed': True}]
 
 
 
-    >>> class MyNewsletterSend(NewsletterSend):
-    ...     def send_send_part(self, send_part):
-    ...         print('sending log %s' % send_part)
-    ...         return True
-    
 
-    >>> ns = MyNewsletterSend()
+
+        >>> import pprint
+        >>> class MyNewsletterSend(NewsletterSend):
+        ...     def send_send_part(self, send_part):
+        ...         print ('sending log ' + pprint.pformat(send_part))
+        ...         return True
+        
+
+        >>> ns = MyNewsletterSend()
 
     TODO: what about html emails?  Attachments?
 
-    >>> newsletter_send_id = ns.create("a subject", "this is the body of the message")
+        >>> newsletter_send_id = ns.create("a subject", "this is the body of the message")
 
-    >>> ns.update_message(newsletter_send_id, "a new subject", "new message body")
-    True
+        >>> ns.update_message(newsletter_send_id, "a new subject", "new message body")
+        True
 
 
-    Some internal testing.
-    >>> a_newsletter_send = ns._get_newsletter_send(newsletter_send_id)
-    
-    >>> a_user = {"email":"asdf2@example.com", "full_name":"Joe blogs", "user_id": "1"}
-    >>> ns._create_send_part(a_newsletter_send, a_user)
-    {'status': 'open', 'body': 'new message body', 'user_id': '1', 'email': 'asdf2@example.com', 'newsletter_send_id': '...', 'full_name': 'Joe blogs', 'id': '...', 'subject': 'a new subject'}
     
 
-    >>> ns.send_test_email(newsletter_send_id, [{"email":"asdf2@example.com", "full_name":"Joe blogs", "user_id": "1"}])
-    sending log {'status': 'open', 'body': 'new message body', 'user_id': '1', 'email': 'asdf2@example.com', 'newsletter_send_id': '...', 'full_name': 'Joe blogs', 'id': '...', 'subject': 'a new subject'}
-    True
-
-
-    >>> ns.add_users(newsletter_send_id,  [{"email":"asdf1@example.com", "full_name":"Joe blogs", "user_id": "1"},
-    ...                          {"email":"asdf2@example.com", "full_name":"Jane blogs", "user_id": "2"}])
-    True
-
-    >>> import pprint
-    >>> len(ns._send_parts.items())
-    2
-
-    >>> pprint.pprint(list(sorted([sp for sp in ns._send_parts.values()])))
-    [{'body': 'new message body',
-      'email': 'asdf1@example.com',
-      'full_name': 'Joe blogs',
-      'id': '...',
-      'newsletter_send_id': '...',
-      'status': 'open',
-      'subject': 'a new subject',
-      'user_id': '1'},
-     {'body': 'new message body',
-      'email': 'asdf2@example.com',
-      'full_name': 'Jane blogs',
-      'id': '...',
-      'newsletter_send_id': '...',
-      'status': 'open',
-      'subject': 'a new subject',
-      'user_id': '2'}]
+    Send a test email.
+        >>> ns.send_test_email(newsletter_send_id, [{"email":"asdf2@example.com", "full_name":"Joe blogs", "user_id": "1"}])
+        sending log {'body': 'new message body',
+         'email': 'asdf2@example.com',
+         'full_name': 'Joe blogs',
+         'id': '...',
+         'newsletter_send_id': '...',
+         'status': 'open',
+         'subject': 'a new subject',
+         'user_id': '1'}
+        True
 
 
 
 
-    >>> ns.send(newsletter_send_id)
-    sending log {'status': 'sending', 'body': 'new message body', 'user_id': '1', 'email': 'asdf1@example.com', 'newsletter_send_id': '...', 'full_name': 'Joe blogs', 'id': '...', 'subject': 'a new subject'}
-    sending log {'status': 'sending', 'body': 'new message body', 'user_id': '2', 'email': 'asdf2@example.com', 'newsletter_send_id': '...', 'full_name': 'Jane blogs', 'id': '...', 'subject': 'a new subject'}
-    True
 
-    >>> ns._newsletter_sends[newsletter_send_id]['status']
-    'finished'
+    Add some users to the send.
+        >>> ns.add_users(newsletter_send_id,  [{"email":"asdf1@example.com", "full_name":"Joe blogs", "user_id": "1"},
+        ...                          {"email":"asdf2@example.com", "full_name":"Jane blogs", "user_id": "2"}])
+        True
 
-    >>> ns.send(newsletter_send_id)
-    True
+
+
+    Check that the users have been added correctly.
+
+        >>> import pprint
+        >>> len(list(ns._send_parts.items()))
+        2
+
+        >>> pprint.pprint(list(sorted([sp for sp in ns._send_parts.values()])))
+        [{'body': 'new message body',
+          'email': 'asdf1@example.com',
+          'full_name': 'Joe blogs',
+          'id': '...',
+          'newsletter_send_id': '...',
+          'status': 'open',
+          'subject': 'a new subject',
+          'user_id': '1'},
+         {'body': 'new message body',
+          'email': 'asdf2@example.com',
+          'full_name': 'Jane blogs',
+          'id': '...',
+          'newsletter_send_id': '...',
+          'status': 'open',
+          'subject': 'a new subject',
+          'user_id': '2'}]
+
+
+
+
+    Send the newsletter.
+
+        >>> ns.send(newsletter_send_id)
+        sending log {'body': 'new message body',
+         'email': 'asdf1@example.com',
+         'full_name': 'Joe blogs',
+         'id': '...',
+         'newsletter_send_id': '...',
+         'status': 'sending',
+         'subject': 'a new subject',
+         'user_id': '1'}
+        sending log {'body': 'new message body',
+         'email': 'asdf2@example.com',
+         'full_name': 'Jane blogs',
+         'id': '...',
+         'newsletter_send_id': '...',
+         'status': 'sending',
+         'subject': 'a new subject',
+         'user_id': '2'}
+        True
+
+        >>> ns._newsletter_sends.get(newsletter_send_id)['status']
+        'finished'
+
+
+    Here we try and send the newletter again, but it does not send anything.
+        >>> ns.send(newsletter_send_id)
+        False
+
+
+
+
+
+
+
+    Testing some internal methods.
+        >>> a_newsletter_send = ns._get_newsletter_send(newsletter_send_id)
+        
+        >>> a_user = {"email":"asdf2@example.com", "full_name":"Joe blogs", "user_id": "1"}
+        >>> ns._create_send_part(a_newsletter_send, a_user)
+        {'status': 'open', 'body': 'new message body', 'user_id': '1', 'email': 'asdf2@example.com', 'newsletter_send_id': '...', 'full_name': 'Joe blogs', 'id': '...', 'subject': 'a new subject'}
 
     """
 
     def __init__(self):
-        self._newsletter_sends = {}
-        self._send_parts = {}
+        self._newsletter_sends = self.make_newsletter_sends_db()
+        self._send_parts = self.make_send_parts_db()
+
+
+    def make_send_parts_db(self):
+        """ make_send_parts_db() returns a dict like object for storing the
+                send_parts.
+        """
+        return {}
+
+    def make_newsletter_sends_db(self):
+        """ make_newsletter_sends_db() returns a dict like object for storing the
+                newsletter_sends.
+        """
+        return {}
+
 
     def send_send_part(self, send_part):
         """ send_send_part(send_part) where the actual email sending happens.
@@ -300,15 +433,18 @@ class NewsletterSend(object):
                     body = body, 
                     created = time.time(),
                     status='open')
-        self._newsletter_sends[uid] = send
+        self._newsletter_sends.update({uid: send})
         return uid
 
     def update_message(self, newsletter_send_id, subject, body):
         """ update_message(newsletter_send_id, subject, body) returns True on a
                successful update.
         """
-        self._newsletter_sends[newsletter_send_id].update(dict(subject = subject, 
-                                                                body=body))
+        a_newsletter_send = self._newsletter_sends.get(newsletter_send_id)
+        a_newsletter_send["subject"] = subject
+        a_newsletter_send["body"] = body
+        self._newsletter_sends.update({newsletter_send_id : a_newsletter_send})
+
         return True
 
 
@@ -316,7 +452,12 @@ class NewsletterSend(object):
         """ send(newsletter_send_id) sends the NewsletterSend with the given
                 newsletter_send_id.  Returns True on success.
         """
-        self._newsletter_sends[newsletter_send_id].update(dict(status = "sending"))
+        a_newsletter_send = self._newsletter_sends.get(newsletter_send_id)
+        if a_newsletter_send.get("status") != "open":
+            return False
+
+        a_newsletter_send["status"] = "sending"
+        self._newsletter_sends.update({newsletter_send_id : a_newsletter_send})
 
         # Send to all of the users attached to this newsletter_send_id
         #           which have not sent already.
@@ -334,7 +475,9 @@ class NewsletterSend(object):
             self._send_parts.update(dict(id = sp))
             pass
 
-        self._newsletter_sends[newsletter_send_id].update(dict(status = "finished"))
+        a_newsletter_send["status"] = "finished"
+        self._newsletter_sends.update({newsletter_send_id : a_newsletter_send})
+        
 
         return True
         
@@ -378,6 +521,7 @@ class NewsletterSend(object):
     def _get_send_parts(self, newsletter_send_id):
         """
         """
+        # we return them sorted, so that they can be repeated.
         return sorted([sp for sp in self._send_parts.values() if sp['newsletter_send_id'] == newsletter_send_id and sp['status'] == 'open'])
 
 
@@ -390,7 +534,7 @@ class NewsletterSend(object):
             id = self._make_uid(),
             newsletter_send_id = newsletter_send['id'],
             user_id = user['user_id'],
-            full_name = user['full_name'],
+            full_name = user.get('full_name', ''),
             email = user['email'],
             body = newsletter_send['body'],
             subject = newsletter_send['subject'],
@@ -410,6 +554,8 @@ class NewsletterSend(object):
 
 
 
+
+
 if __name__ == "__main__":
     import doctest
     doctest.testmod(optionflags = doctest.ELLIPSIS)
